import { ParsedCron, ParsedRate } from './lib/parse';
import { DateInput } from './lib/types';
export * from './lib/local';
export * from './lib/constants';
/**
 * cron is assumed to be validated by AWS already
 * cron is always stored in UTC
 * AWS Cron Expression specs
 * https://docs.aws.amazon.com/AmazonCloudWatch/latest/events/ScheduledEvents.html#CronExpressions
 */
/**
 * Every 5 minutes: rate(5 minutes, 3600000)
 * Every hour: rate(1 hour)
 * Every seven days: rate(7 days)
 */
declare class EventCronParser {
    #private;
    parsedCron: ParsedCron | ParsedRate;
    earliestDate: Date;
    latestDate: Date | null;
    tz: 'local' | 'utc';
    constructor(cron: string, start?: DateInput, end?: DateInput, tz?: 'local' | 'utc');
    next(from?: DateInput, inclusive?: boolean): Date | null;
    setRate(value?: number | undefined, unit?: string | undefined, duration?: undefined, start?: Date, end?: Date | null): void;
    setCron(cron: string, start?: Date, end?: Date | null): void;
    prev(): void;
    range(start: DateInput, end: DateInput, inclusive?: boolean): Date[];
    isInRange(start: DateInput, end: DateInput): boolean | null;
    desc(timezone?: 'local' | 'utc'): string;
    getLocalDays(): number[];
    getUTCMinutes(): string | number;
    getUTCHours(): string | number;
    setUTCHours(hours: number[], minutes?: number[], { preserveLocalDaysOfWeek, referenceDate, }?: {
        preserveLocalDaysOfWeek?: boolean;
        referenceDate?: number | string | Date;
    }): void;
    setDaysOfWeek(daysOfWeek: number[], timezone?: 'local' | 'utc'): number[];
    getCron(): string;
    isRateExpression(): boolean;
    setDuration(duration: number): string;
    validate(): void;
}
export default EventCronParser;
