"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.adjustDateForDST = exports.arrayFindLast = exports.arrayFindFirst = exports.getDaysOfMonthForW = exports.getDaysOfMonthForL = exports.getDaysOfMonthFromDaysOfWeek = void 0;
const isWeekday = (year, month, day) => {
    if (day < 1 || day > 31) {
        return false;
    }
    const thisDate = new Date(year, month - 1, day);
    if (thisDate.getMonth() !== month - 1) {
        return false;
    }
    return thisDate.getDay() > 0 && thisDate.getDay() < 6;
};
const getDaysOfMonthFromDaysOfWeek = (year, month, daysOfWeek) => {
    const daysOfMonth = [];
    let index = 0; // only for "#" use case
    for (let i = 1; i <= 31; i += 1) {
        const thisDate = new Date(year, month - 1, i);
        // already after last day of month
        if (thisDate.getMonth() !== month - 1) {
            break;
        }
        if (daysOfWeek[0] === 'L') {
            if (daysOfWeek[1] === thisDate.getDay() + 1) {
                const sameDayNextWeek = new Date(thisDate.getTime() + 7 * 24 * 3600000);
                if (sameDayNextWeek.getMonth() !== thisDate.getMonth()) {
                    return [i];
                }
            }
        }
        else if (daysOfWeek[0] === '#') {
            if (daysOfWeek[1] === thisDate.getDay() + 1) {
                index += 1;
            }
            if (daysOfWeek[2] === index) {
                return [i];
            }
        }
        else if (daysOfWeek.includes(thisDate.getDay() + 1)) {
            daysOfMonth.push(i);
        }
    }
    return daysOfMonth;
};
exports.getDaysOfMonthFromDaysOfWeek = getDaysOfMonthFromDaysOfWeek;
const getDaysOfMonthForL = (year, month, daysBefore) => {
    for (let i = 31; i >= 28; i -= 1) {
        const thisDate = new Date(year, month - 1, i);
        if (thisDate.getMonth() === month - 1) {
            return [i - daysBefore];
        }
    }
    throw new Error('getDaysOfMonthForL - should not happen');
};
exports.getDaysOfMonthForL = getDaysOfMonthForL;
const getDaysOfMonthForW = (year, month, day) => {
    const offset = [0, 1, -1, 2, -2].find((c) => isWeekday(year, month, day + c));
    if (offset === undefined)
        throw new Error('getDaysOfMonthForW - should not happen');
    return [day + offset];
};
exports.getDaysOfMonthForW = getDaysOfMonthForW;
const arrayFindFirst = (a, f) => {
    return a.find(f);
};
exports.arrayFindFirst = arrayFindFirst;
const arrayFindLast = (a, f) => {
    // note: a.slice().reverse().find(f) is less efficient
    for (let i = a.length - 1; i >= 0; i--) {
        const e = a[i];
        if (f(e)) {
            return e;
        }
    }
};
exports.arrayFindLast = arrayFindLast;
const adjustDateForDST = (date, parsedCron, timezone) => {
    if (timezone === 'local') {
        // check for difference in daylight savings
        let offsetDiff = date.getTimezoneOffset() - new Date(parsedCron.start).getTimezoneOffset();
        if (offsetDiff !== 0) {
            date.setMinutes(date.getMinutes() + offsetDiff);
            return offsetDiff;
        }
    }
    return 0;
};
exports.adjustDateForDST = adjustDateForDST;
//# sourceMappingURL=common.js.map